@@ -1,9 +1,9 @@
 from .misc import *
+from .schema import *
 from .losses import *
 from .models import *
 from .modules import *
 from .trainer import *
-from .protocol import *
 from .constants import *
 from .data import *
 from .pipeline import *
@@ -14,6 +14,7 @@ from .api import ml
 from .api import nlp
 from .api import multimodal
 from .api import api
+from .api.ml.pipeline import MLConfig
 from . import dist
 from . import scripts
 
