@@ -26,17 +26,18 @@ from ..toolkit import no_grad_context
 from ..toolkit import toggle_optimizer
 from ..toolkit import Initializer
 from ...types import losses_type
-from ...protocol import _forward
-from ...protocol import ITrainer
-from ...protocol import StepOutputs
-from ...protocol import ILoss
-from ...protocol import TrainerState
-from ...protocol import IDLModel
-from ...protocol import _IMetric
-from ...protocol import MetricsOutputs
-from ...protocol import WithDeviceMixin
-from ...protocol import IDataLoader
-from ...protocol import ModelWithCustomSteps
+from ...schema import _forward
+from ...schema import ITrainer
+from ...schema import StepOutputs
+from ...schema import ILoss
+from ...schema import TrainerState
+from ...schema import IDLModel
+from ...schema import _IMetric
+from ...schema import MetricsOutputs
+from ...schema import WithDeviceMixin
+from ...schema import IDataLoader
+from ...schema import ModelWithCustomSteps
+from ...schema import TrainerConfig
 from ...constants import INPUT_KEY
 from ...constants import LABEL_KEY
 from ...constants import PREDICTIONS_KEY
@@ -254,7 +255,7 @@ class CustomModule(WithDeviceMixin, nn.Module):
     def init_with_trainer(self, trainer: ITrainer) -> None:
         pass
 
-    def permute_trainer_config(self, trainer_config: Dict[str, Any]) -> None:
+    def permute_trainer_config(self, trainer_config: TrainerConfig) -> None:
         pass
 
 
@@ -440,7 +441,7 @@ def register_custom_module(
             def init_ddp(self) -> None:
                 self.core.init_ddp()
 
-            def permute_trainer_config(self, trainer_config: Dict[str, Any]) -> None:
+            def permute_trainer_config(self, trainer_config: TrainerConfig) -> None:
                 self.core.permute_trainer_config(trainer_config)
 
             def _init_with_trainer(self, trainer: ITrainer) -> None:
