@@ -32,24 +32,24 @@ from torch.distributed.optim import ZeroRedundancyOptimizer as ZeRO
 from torch.utils.data.distributed import DistributedSampler
 
 from .data import DLLoader
-from .protocol import ITrainer
-from .protocol import DeviceInfo
-from .protocol import IDataModule
-from .protocol import StepOutputs
-from .protocol import ILoss
-from .protocol import TqdmSettings
-from .protocol import TrainerState
-from .protocol import IDLModel
-from .protocol import OptimizerPack
-from .protocol import MetricsOutputs
-from .protocol import MonitorResults
-from .protocol import TrainerMonitor
-from .protocol import _IMetric
-from .protocol import _MultipleMetrics
-from .protocol import TrainerCallback
-from .protocol import IInference
-from .protocol import IDataLoader
-from .protocol import ModelWithCustomSteps
+from .schema import ITrainer
+from .schema import DeviceInfo
+from .schema import IDataModule
+from .schema import StepOutputs
+from .schema import ILoss
+from .schema import TqdmSettings
+from .schema import TrainerState
+from .schema import IDLModel
+from .schema import OptimizerPack
+from .schema import MetricsOutputs
+from .schema import MonitorResults
+from .schema import TrainerMonitor
+from .schema import _IMetric
+from .schema import _MultipleMetrics
+from .schema import TrainerCallback
+from .schema import IInference
+from .schema import IDataLoader
+from .schema import ModelWithCustomSteps
 from .constants import LOSS_KEY
 from .constants import PT_PREFIX
 from .constants import SCORES_FILE
